'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _cliEngineHeroku = require('cli-engine-heroku');

var _util = require('util');

var _herokuCliColor = require('heroku-cli-color');

var _herokuCliColor2 = _interopRequireDefault(_herokuCliColor);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

class API extends _cliEngineHeroku.Command {

  async run() {
    let request = { headers: {} };
    let path = this.args.path;
    request.method = this.args.method;
    if (!path) {
      path = request.method;
      request.method = 'GET';
    }
    request.method = request.method.toUpperCase();
    if (!path.startsWith('/')) path = `/${path}`;
    let version = this.flags.version || '3';
    request.headers['accept'] = `application/vnd.heroku+json; version=${version}`;
    if (this.flags['accept-inclusion']) {
      request.headers['Accept-Inclusion'] = this.flags['accept-inclusion'];
    }
    if (request.method === 'PATCH' || request.method === 'PUT' || request.method === 'POST') {
      request.body = await this.getBody();
    }
    const fetch = async (body = []) => {
      this.out.action.start(_herokuCliColor2.default`{cyanBright ${request.method}} ${this.heroku.host}${path}`);
      let response;
      try {
        response = await this.heroku.request(path, request);
      } catch (err) {
        if (!err.http || !err.http.statusCode) throw err;
        this.out.action.stop(_herokuCliColor2.default`{redBright ${err.http.statusCode}}`);
        throw err;
      }
      let msg = _herokuCliColor2.default`{greenBright ${response.response.statusCode}}`;
      if (Array.isArray(response.body)) msg += ` ${response.body.length + body.length} items`;
      this.out.action.stop(msg);
      if (Array.isArray(response.body) && response.response.headers['next-range']) {
        request.headers['range'] = response.response.headers['next-range'];
        return fetch(body.concat(response.body));
      }
      return Array.isArray(response.body) ? body.concat(response.body) : response.body;
    };
    let body = await fetch();
    if (typeof body === 'string') {
      this.out.log(body);
    } else {
      this.out.styledJSON(body);
    }
  }

  async getBody() {
    const getStdin = require('get-stdin');
    const edit = require('edit-string');

    let body = this.flags.body || (await getStdin());
    if (!body) {
      this.out.warn('no stdin provided');
      return;
    }
    try {
      return JSON.parse(body);
    } catch (e) {
      let err = new Error(`Request body must be valid JSON
Received:
${(0, _util.inspect)(body)}`);
      if (process.stdin.isTTY) {
        this.out.warn(err);
        return JSON.parse((await edit(body, { postfix: '.json' })));
      } else throw new Error(err);
    }
  }
}
exports.default = API;
API.topic = 'api';
API.description = 'make a manual API request';
API.flags = {
  version: _cliEngineHeroku.flags.string({ char: 'v', description: 'version to use (e.g. 2, 3, or 3.variant)' }),
  'accept-inclusion': _cliEngineHeroku.flags.string({ char: 'a', description: 'Accept-Inclusion header to use' }),
  body: _cliEngineHeroku.flags.string({ char: 'b', description: 'JSON input body' })
};
API.args = [{ name: 'method', description: 'GET, POST, PUT, PATCH, or DELETE' }, { name: 'path', description: 'endpoint to call', optional: true }];
API.help = `The api command is a convenient but low-level way to send requests
to the Heroku API. It sends an HTTP request to the Heroku API
using the given method on the given path. For methods PUT, PATCH,
and POST, it uses stdin unmodified as the request body. It prints
the response unmodified on stdout.

It is essentially like curl for the Heroku API.

Method name input will be upcased, so both 'heroku api GET /apps' and
'heroku api get /apps' are valid commands.

Examples:

    $ heroku api GET /apps/myapp
    {
      created_at: "2011-11-11T04:17:13-00:00",
      id: "12345678-9abc-def0-1234-456789012345",
      name: "myapp",
      …
    }

    $ heroku api PATCH /apps/myapp/config-vars --body '{"FOO": "bar"}'
    {
      FOO: "bar"
      …
    }

    $ export HEROKU_HEADERS
    $ HEROKU_HEADERS='
    Content-Type: application/x-www-form-urlencoded
    Accept: application/json
    '
    $ printf 'type=web&qty=2' | heroku api POST /apps/myapp/ps/scale
    2
`;